from matplotlib import _api
from matplotlib.backends import backend_webagg_core


class FigureCanvasFletAgg(backend_webagg_core.FigureCanvasWebAggCore):
    manager_class = _api.classproperty(lambda cls: FigureManagerFletAgg)
    supports_blit = False


class FigureManagerFletAgg(backend_webagg_core.FigureManagerWebAgg):
    _toolbar2_class = backend_webagg_core.NavigationToolbar2WebAgg


FigureCanvas = FigureCanvasFletAgg
FigureManager = FigureManagerFletAgg
interactive = True



from dataclasses import dataclass, field
from typing import Any, Optional

import flet as ft
from flet_charts.chart_axis import ChartAxis
from flet_charts.line_chart_data import LineChartData
from flet_charts.types import ChartEventType, ChartGridLines, HorizontalAlignment

__all__ = [
    "LineChart",
    "LineChartEvent",
    "LineChartEventSpot",
    "LineChartTooltip",
]


@dataclass
class LineChartEventSpot:
    bar_index: int
    """
    The line's index or `-1` if no line was hovered.
    """

    spot_index: int
    """
    The line's point index or `-1` if no point was hovered.
    """

    def copy(
        self,
        *,
        bar_index: Optional[int] = None,
        spot_index: Optional[int] = None,
    ) -> "LineChartEventSpot":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return LineChartEventSpot(
            bar_index=bar_index if bar_index is not None else self.bar_index,
            spot_index=spot_index if spot_index is not None else self.spot_index,
        )


@dataclass
class LineChartEvent(ft.Event["LineChart"]):
    type: ChartEventType
    """
    The type of event that occured.
    """

    spots: list[LineChartEventSpot]
    """
    Spots on which the event occurred.
    """


@dataclass
class LineChartTooltip:
    """Configuration of the tooltip for [`LineChart`][(p).]s."""

    bgcolor: ft.ColorValue = "#FF607D8B"
    """
    Background color of tooltip.
    """

    border_radius: Optional[ft.BorderRadiusValue] = None
    """
    The tooltip's border radius.
    """

    margin: ft.Number = 16
    """
    Applies a bottom margin for showing tooltip on top of rods.
    """

    padding: ft.PaddingValue = field(
        default_factory=lambda: ft.Padding.symmetric(vertical=8, horizontal=16)
    )
    """
    Applies a padding for showing contents inside the tooltip.
    """

    max_width: ft.Number = 120
    """
    Restricts the tooltip's width.
    """

    rotation: ft.Number = 0.0
    """
    The tooltip's rotation angle in degrees.
    """

    horizontal_offset: ft.Number = 0.0
    """
    Applies horizontal offset for showing tooltip.
    """

    border_side: ft.BorderSide = field(default_factory=lambda: ft.BorderSide.none())
    """
    Defines the borders of this tooltip.
    """

    fit_inside_horizontally: bool = False
    """
    Forces the tooltip to shift horizontally inside the chart, if overflow happens.
    """

    fit_inside_vertically: bool = False
    """
    Forces the tooltip to shift vertically inside the chart, if overflow happens.
    """

    show_on_top_of_chart_box_area: bool = False
    """
    Whether to force the tooltip container to top of the line.
    """

    horizontal_alignment: HorizontalAlignment = HorizontalAlignment.CENTER
    """
    The horizontal alignment of this tooltip.
    """

    def copy(
        self,
        *,
        bgcolor: Optional[ft.ColorValue] = None,
        border_radius: Optional[ft.BorderRadiusValue] = None,
        margin: Optional[ft.Number] = None,
        padding: Optional[ft.PaddingValue] = None,
        max_width: Optional[ft.Number] = None,
        rotation: Optional[ft.Number] = None,
        horizontal_offset: Optional[ft.Number] = None,
        border_side: Optional[ft.BorderSide] = None,
        fit_inside_horizontally: Optional[bool] = None,
        fit_inside_vertically: Optional[bool] = None,
        show_on_top_of_chart_box_area: Optional[bool] = None,
    ) -> "LineChartTooltip":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return LineChartTooltip(
            bgcolor=bgcolor if bgcolor is not None else self.bgcolor,
            border_radius=(
                border_radius if border_radius is not None else self.border_radius
            ),
            margin=margin if margin is not None else self.margin,
            padding=padding if padding is not None else self.padding,
            max_width=max_width if max_width is not None else self.max_width,
            rotation=rotation if rotation is not None else self.rotation,
            horizontal_offset=(
                horizontal_offset
                if horizontal_offset is not None
                else self.horizontal_offset
            ),
            border_side=border_side if border_side is not None else self.border_side,
            fit_inside_horizontally=(
                fit_inside_horizontally
                if fit_inside_horizontally is not None
                else self.fit_inside_horizontally
            ),
            fit_inside_vertically=(
                fit_inside_vertically
                if fit_inside_vertically is not None
                else self.fit_inside_vertically
            ),
            show_on_top_of_chart_box_area=(
                show_on_top_of_chart_box_area
                if show_on_top_of_chart_box_area is not None
                else self.show_on_top_of_chart_box_area
            ),
        )


@ft.control("LineChart")
class LineChart(ft.LayoutControl):
    """
    Draws a line chart.

    ```python
    fch.LineChart(
        data_series=[
            fch.LineChartData(
                color=ft.Colors.BLUE_GREY_500,
                curved=True,
                points=[
                    fch.LineChartDataPoint(1, 0.5),
                    fch.LineChartDataPoint(2, 1.5),
                    fch.LineChartDataPoint(3, 1),
                ],
            ),
            ...
        ],
        min_y=0,
        max_y=3,
        min_x=0,
        max_x=5,
    )
    ```

    """

    data_series: list[LineChartData] = field(default_factory=list)
    """
    A list of [`LineChartData`][(p).]
    controls drawn as separate lines on a chart.
    """

    animation: ft.AnimationValue = field(
        default_factory=lambda: ft.Animation(
            duration=ft.Duration(milliseconds=150), curve=ft.AnimationCurve.LINEAR
        )
    )
    """
    Controls chart implicit animation.
    """

    interactive: bool = True
    """
    Enables automatic tooltips and points highlighting when hovering over the chart.
    """

    point_line_start: Optional[ft.Number] = None
    """
    The start of the vertical line drawn under the selected point.

    Defaults to chart's bottom edge.
    """

    point_line_end: Optional[ft.Number] = None
    """
    The end of the vertical line drawn at selected point position.

    Defaults to data point's `y` value.
    """

    bgcolor: Optional[ft.ColorValue] = None
    """
    Background color of the chart.
    """

    border: Optional[ft.Border] = None
    """
    The border around the chart.
    """

    horizontal_grid_lines: Optional[ChartGridLines] = None
    """
    Controls drawing of chart's horizontal lines.
    """

    vertical_grid_lines: Optional[ChartGridLines] = None
    """
    Controls drawing of chart's vertical lines.
    """

    left_axis: Optional[ChartAxis] = None
    """
    Defines the appearance of the left axis, its title and labels.
    """

    top_axis: Optional[ChartAxis] = None
    """
    Defines the appearance of the top axis, its title and labels.
    """

    right_axis: Optional[ChartAxis] = None
    """
    Defines the appearance of the right axis, its title and labels.
    """

    bottom_axis: Optional[ChartAxis] = None
    """
    Defines the appearance of the bottom axis, its title and labels.
    """

    baseline_x: Optional[ft.Number] = None
    """
    Baseline value for X axis.
    """

    min_x: Optional[ft.Number] = None
    """
    Defines the minimum displayed value for X axis.
    """

    max_x: Optional[ft.Number] = None
    """
    Defines the maximum displayed value for X axis.
    """

    baseline_y: Optional[ft.Number] = None
    """
    Baseline value for Y axis.
    """

    min_y: Optional[ft.Number] = None
    """
    Defines the minimum displayed value for Y axis.
    """

    max_y: Optional[ft.Number] = None
    """
    Defines the maximum displayed value for Y axis.
    """

    tooltip: Optional[LineChartTooltip] = field(
        default_factory=lambda: LineChartTooltip()
    )
    """
    The tooltip configuration for this chart.

    If set to `None`, no tooltips will be shown throughout this chart.
    """

    on_event: Optional[ft.EventHandler[LineChartEvent]] = None
    """
    Fires when a chart line is hovered or clicked.
    """

    def __post_init__(self, ref: Optional[ft.Ref[Any]]):
        super().__post_init__(ref)
        self._internals["skip_properties"] = ["tooltip"]
        self._internals["skip_inherited_notifier"] = True



from dataclasses import dataclass, field
from typing import Any, Optional, Union

import flet as ft
from flet_charts.types import ChartDataPointTooltip, ChartPointShape

__all__ = ["ScatterChartSpot", "ScatterChartSpotTooltip"]


@dataclass
class ScatterChartSpotTooltip(ChartDataPointTooltip):
    """
    Tooltip configuration for the [`ScatterChartSpot`][(p).].
    """

    text: Optional[str] = None
    """
    The text to display in the tooltip.

    When `None`, defaults to [`ScatterChartSpot.y`][(p).].
    """

    bottom_margin: ft.Number = 8
    """
    The bottom space from the spot.
    """

    def copy(
        self,
        *,
        text: Optional[str] = None,
        text_style: Optional[ft.TextStyle] = None,
        text_align: Optional[ft.TextAlign] = None,
        text_spans: Optional[list[ft.TextSpan]] = None,
        rtl: Optional[bool] = None,
        bottom_margin: Optional[float] = None,
    ) -> "ScatterChartSpotTooltip":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return ScatterChartSpotTooltip(
            text=text if text is not None else self.text,
            text_style=text_style if text_style is not None else self.text_style,
            text_align=text_align if text_align is not None else self.text_align,
            text_spans=text_spans.copy()
            if text_spans is not None
            else (self.text_spans.copy() if self.text_spans is not None else None),
            rtl=rtl if rtl is not None else self.rtl,
            bottom_margin=bottom_margin
            if bottom_margin is not None
            else self.bottom_margin,
        )


@ft.control("ScatterChartSpot")
class ScatterChartSpot(ft.BaseControl):
    """A spot on a scatter chart."""

    x: Optional[ft.Number] = None
    """
    The position of a spot on `X` axis.
    """

    y: Optional[ft.Number] = None
    """
    The position of a spot on `Y` axis.
    """

    visible: bool = True
    """
    Determines wether to show or hide the spot.
    """

    radius: Optional[ft.Number] = None
    """
    Radius of a spot.
    """

    color: Optional[ft.ColorValue] = None
    """
    Color of a spot.
    """

    render_priority: ft.Number = 0
    """
    Sort by this to manage overlap.
    """

    x_error: Optional[Any] = None
    """
    Determines the error range of the data point using
    [FlErrorRange](https://github.com/imaNNeo/fl_chart/blob/main/repo_files/documentations/base_chart.md#flerrorrange)
    (which contains lowerBy and upperValue) for the `X` axis.
    """

    y_error: Optional[Any] = None
    """
    Determines the error range of the data point using
    [FlErrorRange](https://github.com/imaNNeo/fl_chart/blob/main/repo_files/documentations/base_chart.md#flerrorrange)
    (which contains lowerBy and upperValue) for the `Y` axis.
    """

    selected: bool = False
    """
    Whether to treat this spot as selected.
    """

    tooltip: Union[ScatterChartSpotTooltip, str] = field(
        default_factory=lambda: ScatterChartSpotTooltip()
    )
    """
    Tooltip configuration for this spot.
    """

    show_tooltip: bool = True
    """
    Wether to show the tooltip.
    """

    label_text: str = ""
    """
    TBD
    """

    label_text_style: ft.TextStyle = field(default_factory=lambda: ft.TextStyle())
    """
    TBD
    """

    point: Union[None, bool, ChartPointShape] = None
    """
    TBD
    """

    def before_update(self):
        super().before_update()
        self._internals["tooltip"] = (
            ScatterChartSpotTooltip(text=self.tooltip)
            if isinstance(self.tooltip, str)
            else self.tooltip
        )



from dataclasses import dataclass, field
from typing import Any, Optional

import flet as ft
from flet_charts.candlestick_chart_spot import CandlestickChartSpot
from flet_charts.chart_axis import ChartAxis
from flet_charts.types import ChartEventType, ChartGridLines, HorizontalAlignment

__all__ = [
    "CandlestickChart",
    "CandlestickChartEvent",
    "CandlestickChartTooltip",
]


@dataclass
class CandlestickChartTooltip:
    """Configuration of the tooltip for [`CandlestickChart`][(p).]s."""

    bgcolor: ft.ColorValue = "#FFFFECEF"
    """
    Background color applied to the tooltip bubble.
    """

    border_radius: ft.BorderRadiusValue = field(
        default_factory=lambda: ft.BorderRadius.all(4)
    )
    """
    Corner radius of the tooltip bubble.
    """

    padding: ft.PaddingValue = field(
        default_factory=lambda: ft.Padding.symmetric(vertical=8, horizontal=16)
    )
    """
    Padding inside the tooltip bubble.
    """

    max_width: ft.Number = 120
    """
    Maximum width of the tooltip bubble.
    """

    rotation: ft.Number = 0.0
    """
    Rotation angle (in degrees) applied to the tooltip bubble.
    """

    horizontal_offset: ft.Number = 0
    """
    Horizontal offset applied to the tooltip bubble.
    """

    horizontal_alignment: HorizontalAlignment = HorizontalAlignment.CENTER
    """
    Horizontal alignment of the tooltip relative to the tapped candlestick.
    """

    border_side: ft.BorderSide = field(default_factory=lambda: ft.BorderSide.none())
    """
    The tooltip bubble border.
    """

    fit_inside_horizontally: bool = False
    """
    Forces the tooltip bubble to remain inside the chart horizontally.
    """

    fit_inside_vertically: bool = False
    """
    Forces the tooltip bubble to remain inside the chart vertically.
    """

    show_on_top_of_chart_box_area: bool = False
    """
    When set to `True`, the tooltip is drawn at the top of the chart box.
    """

    def copy(
        self,
        *,
        bgcolor: Optional[ft.ColorValue] = None,
        border_radius: Optional[ft.BorderRadiusValue] = None,
        padding: Optional[ft.PaddingValue] = None,
        max_width: Optional[ft.Number] = None,
        rotation: Optional[ft.Number] = None,
        horizontal_offset: Optional[ft.Number] = None,
        horizontal_alignment: Optional[HorizontalAlignment] = None,
        border_side: Optional[ft.BorderSide] = None,
        fit_inside_horizontally: Optional[bool] = None,
        fit_inside_vertically: Optional[bool] = None,
        show_on_top_of_chart_box_area: Optional[bool] = None,
    ) -> "CandlestickChartTooltip":
        """
        Returns a copy of this object with the specified properties overridden.
        """
        return CandlestickChartTooltip(
            bgcolor=bgcolor if bgcolor is not None else self.bgcolor,
            border_radius=(
                border_radius if border_radius is not None else self.border_radius
            ),
            padding=padding if padding is not None else self.padding,
            max_width=max_width if max_width is not None else self.max_width,
            rotation=rotation if rotation is not None else self.rotation,
            horizontal_offset=(
                horizontal_offset
                if horizontal_offset is not None
                else self.horizontal_offset
            ),
            horizontal_alignment=(
                horizontal_alignment
                if horizontal_alignment is not None
                else self.horizontal_alignment
            ),
            border_side=border_side if border_side is not None else self.border_side,
            fit_inside_horizontally=(
                fit_inside_horizontally
                if fit_inside_horizontally is not None
                else self.fit_inside_horizontally
            ),
            fit_inside_vertically=(
                fit_inside_vertically
                if fit_inside_vertically is not None
                else self.fit_inside_vertically
            ),
            show_on_top_of_chart_box_area=(
                show_on_top_of_chart_box_area
                if show_on_top_of_chart_box_area is not None
                else self.show_on_top_of_chart_box_area
            ),
        )


@dataclass
class CandlestickChartEvent(ft.Event["CandlestickChart"]):
    """Event raised for interactions with a [`CandlestickChart`][(p).]."""

    type: ChartEventType
    """
    Type of pointer gesture that triggered the event.
    """

    spot_index: Optional[int] = None
    """
    Index of the candlestick that was interacted with; `None` if none.
    """


@ft.control("CandlestickChart")
class CandlestickChart(ft.LayoutControl):
    """
    Draws a candlestick chart representing OHLC values.

    ```python
    fch.CandlestickChart(
        min_x=-1,
        max_x=2,
        min_y=20,
        max_y=30,
        bgcolor=ft.Colors.AMBER_200,
        spots=[
            fch.CandlestickChartSpot(
                x=0,
                open=22.6,
                high=28.3,
                low=21.4,
                close=24.1,
            ),
            ...
        ],
    )
    ```

    """

    spots: list[CandlestickChartSpot] = field(default_factory=list)
    """
    Candlesticks to display on the chart.
    """

    animation: ft.AnimationValue = field(
        default_factory=lambda: ft.Animation(
            duration=ft.Duration(milliseconds=150), curve=ft.AnimationCurve.LINEAR
        )
    )
    """
    Controls chart implicit animations.
    """

    interactive: bool = True
    """
    Enables automatic tooltips and highlighting when hovering the chart.
    """

    show_tooltips_for_selected_spots_only: bool = False
    """
    Whether to permanently and only show the tooltips of spots with their
    [`selected`][(p).CandlestickChartSpot.selected] property set to `True`.
    """

    long_press_duration: Optional[ft.DurationValue] = None
    """
    The duration of a long press on the chart.
    """

    touch_spot_threshold: ft.Number = 4
    """
    The distance threshold to consider a touch near a candlestick.
    """

    bgcolor: Optional[ft.ColorValue] = None
    """
    Background color of the chart.
    """

    border: Optional[ft.Border] = None
    """
    Border drawn around the chart.
    """

    horizontal_grid_lines: Optional[ChartGridLines] = None
    """
    Horizontal grid lines configuration.
    """

    vertical_grid_lines: Optional[ChartGridLines] = None
    """
    Vertical grid lines configuration.
    """

    left_axis: Optional[ChartAxis] = None
    """
    Appearance of the left axis, its title and labels.
    """

    top_axis: Optional[ChartAxis] = None
    """
    Appearance of the top axis, its title and labels.
    """

    right_axis: Optional[ChartAxis] = None
    """
    Appearance of the right axis, its title and labels.
    """

    bottom_axis: Optional[ChartAxis] = None
    """
    Appearance of the bottom axis, its title and labels.
    """

    baseline_x: Optional[ft.Number] = None
    """
    Baseline value on the X axis.
    """

    min_x: Optional[ft.Number] = None
    """
    Minimum value displayed on the X axis.
    """

    max_x: Optional[ft.Number] = None
    """
    Maximum value displayed on the X axis.
    """

    baseline_y: Optional[ft.Number] = None
    """
    Baseline value on the Y axis.
    """

    min_y: Optional[ft.Number] = None
    """
    Minimum value displayed on the Y axis.
    """

    max_y: Optional[ft.Number] = None
    """
    Maximum value displayed on the Y axis.
    """

    rotation_quarter_turns: ft.Number = 0
    """
    Number of quarter turns (90-degree increments) to rotate the chart.
    """

    tooltip: Optional[CandlestickChartTooltip] = field(
        default_factory=lambda: CandlestickChartTooltip()
    )
    """
    Tooltip configuration for the chart.
    """

    on_event: Optional[ft.EventHandler[CandlestickChartEvent]] = None
    """
    Called when an event occurs on this chart.
    """

    def __post_init__(self, ref: Optional[ft.Ref[Any]]):
        super().__post_init__(ref)
        self._internals["skip_properties"] = ["tooltip"]
